@@ -17,6 +17,8 @@ List ivx_fit_cpp(const arma::vec & y, const arma::mat & X, int K = 1) {
   int  nn = xlag.n_rows, l = xlag.n_cols;
   // NumericVector df(2); df(0) = l, df(1) = nn -l;
 
+  ////////////////////////////////////////////////
+
   //join_horiz to include intercept
   arma::mat Xols = join_rows(ones(nr-1, 1), xlag);
 
@@ -27,6 +29,8 @@ List ivx_fit_cpp(const arma::vec & y, const arma::mat & X, int K = 1) {
   arma::colvec std_err = arma::sqrt(s2 * arma::diagvec(arma::pinv(arma::trans(Xols)*Xols)));
   arma::colvec tstat = Aols/std_err;
 
+  ////////////////////////////////////////////////
+
   arma::mat Rn = zeros<mat>(l, l);
   for (int i = 0; i < l; i++) {
     double coef_Rn = as_scalar(arma::solve(xlag.col(i), xt.col(i)));
@@ -82,7 +86,8 @@ List ivx_fit_cpp(const arma::vec & y, const arma::mat & X, int K = 1) {
   arma::mat residue = sum(q)/nn;
   arma::mat Omegaeu = covuhat + residue.t();
 
-  // instrument construction
+  ////////// instrument construction ////////////
+
   arma::mat Rz = (1-1/(pow(nn, 0.95)))*eye(l,l);
   arma::mat diffx = xt - xlag;
   arma::mat z = zeros<mat>(nn,l);
@@ -101,6 +106,8 @@ List ivx_fit_cpp(const arma::vec & y, const arma::mat & X, int K = 1) {
   }
   arma::mat meanzK = mean(ZK);
 
+  ////////////////////////////////////////////////
+
   arma::vec yy = zeros<vec>(n);
   for (int i = 0; i < n; ++i){
     yy.row(i)=sum(yt.rows(i, i+K-1));
@@ -121,10 +128,14 @@ List ivx_fit_cpp(const arma::vec & y, const arma::mat & X, int K = 1) {
   ////////////////////////////////////////////////
 
   arma::mat Aivx = Yt.t()*Z*pinv(Xt.t()*Z);
+  arma::colvec fitted =  Xt*trans(Aivx);
   arma::mat intercept = mean(Yt) - mean(Xt) * Aivx.t();
-  arma::colvec fitted = Xt*trans(Aivx);
   arma::colvec residuals = Yt - fitted;
 
+  ///////////////// No demeaning /////////////////
+  arma::mat interceptm = mean(y) - mean(xlag) * Aivx.t();
+  arma::colvec fittedm = as_scalar(interceptm) + xlag * trans(Aivx);
+
   arma::mat FM = covepshat - Omegaeu.t()*inv(Omegauu)*Omegaeu;
   arma::mat M = ZK.t()*ZK*as_scalar(covepshat)-n*meanzK.t()*meanzK*as_scalar(FM);
   arma::mat H = eye<mat>(l,l);
@@ -137,6 +148,29 @@ List ivx_fit_cpp(const arma::vec & y, const arma::mat & X, int K = 1) {
 
   ////////////////////////////////////////////////
 
+  List data = List::create(
+    _("X") = xlag,
+    _("y") = yt.col(0)
+    );
+
+  List initial = List::create(
+    _("intercept") = interceptm,
+    _("fitted") = fittedm
+    );
+
+  List datam = List::create(
+    _("X") = Xt,
+    _("y") = Yt.col(0)
+  );
+
+  List ols = List::create(
+    _("Aols") = Aols,
+    _("se") = std_err,
+    _("tstat_ols") = tstat,
+    _("residuals_ols") = epshat,
+    _("rank_ols") = rank(X)
+    );
+
   return List::create(
     _("Aivx") = Aivx.t(),
     _("intercept") = intercept,
@@ -145,18 +179,19 @@ List ivx_fit_cpp(const arma::vec & y, const arma::mat & X, int K = 1) {
     _("wivx") = wivx,
     _("wivxind") = wivxind,
     _("zinvxind") = wivxind_z,
-    _("Aols") = Aols,
-    _("tstat_ols") = tstat,
-    _("residuals_ols") = epshat,
     _("rank") = rank(Xt),
-    _("rank_ols") = rank(X),
     _("horizons") = K,
     _("df.residuals") = nn - l,
     _("df") = l,
     _("delta") = corrmat,
     _("Rn") = diagvec(Rn),
     _("Rz") = diagvec(Rz),
-    _("varcov") = Q
+    _("varcov") = Q,
+    _("ols") = ols,
+    _("data") = data,
+    _("initial") = initial,
+    _("datam") = datam
   );
 
 }
+
